;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x419f2e2bb079f2fd23cd2cd3ffd2aeb8 . "./code.cg2js/cg-common-control.fasl")
(#xe85c368d1721aedca576afa03b8772b7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x09877abc271232d3769367c21c83322e . "./code.cg2js/cg-yes-no-string.fasl")
(#x78907906ccb3bd6ce447e134bface02b . "./code.cg2js/cg-clipboard.fasl")
(#xebc19a647d2825998d40f9e0d20535a3 . "./code.cg2js/cg-dde-utils.fasl")
(#xb4574626e01671597868d87ee8ef62cc . "./code.cg2js/cg-editable-text.fasl")
(#x8cfcc1c3cccdb036b47959c9d3361513 . "./code.cg2js/cg-player.fasl")
(#x1d4f4898b0bc677044133e6058c69ee8 . "./code.cg2js/cg-text-or-combo.fasl")
(#xd0e5a3d4073fe2cf53ddfcf0329e7f9a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xdf821cad06ff625b5a380a263f1f9f78 . "./code.cg2js/cg-graphics-context.fasl")
(#xc367228ce4f8a62e0937e9793191813b . "./code.cg2js/cg-choose-printer.fasl")
(#xdf6e90837a25ed38d7c69f3776887993 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xc1dd067d6294894d7a8de523b1f349ab . "./code.cg2js/cg-edit-in-place.fasl")
(#xb499243d6b5b905d1b4ef2b4f5a4755a . "./code.cg2js/cg-split-bar.fasl")
(#x9aedbabc27f10cb08a91488fd4bec872 . "./code.cg2js/cg-timer.fasl")
(#x8d869f486fd6e30928ade4375c2cd7ab . "./code.cg2js/cg-progress-indicator.fasl")
(#xab875321be6433f2adf5adf1664abf3d . "./code.cg2js/cg-object-editor.fasl")
(#x5d666e50ec3fb98969069d73ec9b83e0 . "./code.cg2js/cg-fill-texture.fasl")
(#xcc69758daaeb3f68288cdfbb0770b38b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x5b5ac3922902b80105e75df31b64fb7d . "./code.cg2js/cg-os-window.fasl")
(#x602082bce06452ab8cc473897d5f1324 . "./code.cg2js/cg-color-dialog.fasl")
(#xfb89957eeb3a2607530fff3c280c0c3c . "./code.cg2js/cg-combo-box.fasl")
(#x8e29392d947763a7c19bc53d5962afdb . "./code.cg2js/cg-directory-dialog.fasl")
(#xf33d01f5455a17bcca5c267d2d203916 . "./code.cg2js/cg-tooltip.fasl")
(#x773da7ac373976a568b4eba92afada8f . "./code.cg2js/cg-lettered-menu.fasl")
(#xb8f0c6bb25709c6be1d111c0190a1476 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x71581c7cdeeef3b5cacd3c3c317a08e8 . "./code.cg2js/cg-pixmap.fasl")
(#x072646cb39715b53739a30720541b347 . "./code.cg2js/cg.fasl")
(#x6a4e370d2caf4c9f46c2e01d5f17eef3 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa4e7ce141607bb6f901d6f0d0e4d3509 . "./code.cg2js/cg-template-string.fasl")
(#xd5e0edb65fb3dde93f63d2baa4456e1b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x68621408fd8f226db7120c2b0ae11954 . "./code.cg2js/cg-check-box.fasl")
(#xaf8886b22682ab171cb795ace4ea7893 . "./code.cg2js/cg-toggling-widget.fasl")
(#xf330f5880111d0e2511d50a5239f2ac1 . "./code.cg2js/cg-radio-button.fasl")
(#x040ccc1a63397516bb20d1f89f5f2a31 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xc6a9dfc6f7ce69fc1a4a841bd9ab3ca5 . "./code.cg2js/cg-scaling-stream.fasl")
(#xd95534cecffc6970cf748ef0f90267c3 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc7f5ec2215aae91248084eeaca78b848 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2862fd1ca37105e75b05d9e101ea7d7b . "./code.cg2js/cg-grid-widget.fasl")
(#x91f70bdd1f0152f7aa81efc0ac330a1a . "./code.cg2js/cg-text-edit-pane.fasl")
(#x5e7afe58592c9340c78291fec4658895 . "./code.cg2js/cg-item-list.fasl")
(#xdb6e8185092163e0ca7c9c1ef014812f . "./code.cg2js/cg-paren-matching.fasl")
(#x6a0145073b293f0b34a0baf737da02ee . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xecf9bfa31d8c12fcbf0995e23aa0f0fe . "./code.cg2js/cg-html-dialog.fasl")
(#x96f61b9664f8010af75fd8cd18391d74 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf8b907b192a0dd5d5520239125740430 . "./code.cg2js/cg-project-window.fasl")
(#xd163641bec009ac02b7934e5816a603e . "./code.cg2js/cg-toolbar.fasl")
(#x6f47a586faaea5512a541143fb2622bb . "./code.cg2js/cg-caret.fasl")
(#xb7881dc0eaaa3493e3b053fa7a160334 . "./code.cg2js/cg-scroll-bar.fasl")
(#xb9fc1265968d254f32ad0e0d52758a07 . "./code.cg2js/cg-web-browser.fasl")
(#xaed20c9e9f3af01e90327a91f825ee01 . "./code.cg2js/cg-dropping-outline.fasl")
(#xd76663a197dddb63b5090dddb93dc863 . "./code.cg2js/cg-font-dialog.fasl")
(#x80cc6263b40a7706992674d99da974c7 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x66754ff75f0d4009570f739768d072f3 . "./code.cg2js/cg-yes-no-list.fasl")
(#x8d6346a657f88b88a39eddf0d140e4bd . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xedc8e731b26c88a761b2c3e660d93263 . "./code.cg2js/cg-lamp.fasl")
(#x5620ef41a3532862e83494efdff05055 . "./code.cg2js/cg-drawable.fasl")
(#x48ff5cf7a75c53d1ce8a491e7ea3bb52 . "./code.cg2js/cg-choice-list.fasl")
(#x195e097846ba04f178c4a3bd8bbe0526 . "./code.cg2js/cg-curve.fasl")
(#xefd3b9abaa583a060add9585c5abd20e . "./code.cg2js/cg-base.fasl")
(#xaf902e9758c0937c0913648543aeaf72 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xbb89b4b00b4eaf45aef2826444991266 . "./code.cg2js/cg-printing.fasl")
(#x790ecb525cdbabb716e44209a2e37a3d . "./code.cg2js/cg-chart-or-plot.fasl")
(#x599d82a9dae5290d00690662f7e06977 . "./code.cg2js/cg-hotspot.fasl")
(#x090c1d0e77b7beced8fcda9bc48ac7bf . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb86112b8768fc6a2f70f05b3c66358ff . "./code.cg2js/cg-utility-dialog.fasl")
(#x7321659b2238d0d91367ca09594be7ff . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3958b2be896109243bf40016c43dfc78 . "./code.cg2js/cg-comtab.fasl")
(#xb9fb8bc79301d0c364dbbbe045b9e1a1 . "./code.cg2js/cg-os-widget.fasl")
(#xf7b24ebf71b7271afbd7b479d480ef4b . "./code.cg2js/cg-icon.fasl")
(#x83019b7f81f6a53261a793940c4cd6ed . "./code.cg2js/cg-dev.fasl")
(#x4d516dca7fe53a2924d9cddc74ceb350 . "./code.cg2js/cg-calendar.fasl")
(#xcbb2ac35160ecf938a64c6136637e153 . "./code.cg2js/cg-group-box.fasl")
(#xaa36439f45e5478a36db891f44802060 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xbe0f170a52b91ebcf1d46dd534ad5991 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7a808fdd625ced8f25cd0dbb865c60e5 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x07ed7eaddd7408938e78590122584cc6 . "./code.cg2js/cg-nodes.fasl")
(#x085fde121f700fd3a9a1e864d535f85c . "./code.cg2js/cg-intersect.fasl")
(#xbbf07fb5c4305d40cc9e88eac901630f . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x3de5a8b91c1b6c05556641c62a692607 . "./code.cg2js/cg-class-support.fasl")
(#x00fffe931214427c8c0db3578ed5f992 . "./code.cg2js/cg-dialog-item.fasl")
(#xa0f105695f6fb168a65e8d869390af3b . "./code.cg2js/cg-plot-widget.fasl")
(#x11bba2c598cc58e7ced5e93ed6f4fa48 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6894f4c255939b620a490f4b01b67695 . "./code.cg2js/cg-up-down-control.fasl")
(#x7c2c185e7d7cd86dba1c771873cca371 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc59425fba5779d1bd168f72afdc1b034 . "./code.cg2js/cg-status-bar.fasl")
(#xaab9e4de00385704db352420c3e25ad0 . "./code.cg2js/cg-class-grid.fasl")
(#x3d9e85055c8e99cfb48f1d80abce5bac . "./code.cg2js/cg-wrap-string.fasl")
(#xe7466697850f5a80211d2162eab8a5b2 . "./code.cg2js/cg-layout.fasl")
(#x2a69880c97eefad56f68e48ce380a233 . "./code.cg2js/cg-lisp-widget.fasl")
(#xe4e6ddf3f72bccecf295b55d7a2c40b6 . "./code.cg2js/cg-button.fasl")
(#x448295dd272fcb4d4d24caaf7802c234 . "./code.cg2js/cg-string-dialog.fasl")
(#xe2d3a72a9f9c35ba7a8b3baa3c3ca66e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x0631a85095f34f82fde441babab6c9ce . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf1db34f3d34291e26a2f1bf14c13f39f . "./code.cg2js/cg-bitmap-stream.fasl")
(#x40fa7023247c4f930e7cc12005fe7d36 . "./code.cg2js/cg-static-text.fasl")
(#xb74273fc82240442edb316aafc15de36 . "./code.cg2js/cg-outline.fasl")
(#x3709f50f962ed648b3f9f8af87f174e0 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4c0c9d97fa826ee74c18c05ec9f22662 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7f32c6c51270a3e57d306a0f5ec89da8 . "./code.cg2js/cg-picture-widget.fasl")
(#x68cf695d5ab1d7888c33df076a2e7d57 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xbf3c14519dae6077c54e68e38edca823 . "./code.cg2js/cg-chart-widget.fasl")
(#x4379a78c586e88755b60a3585f9b8a0e . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xff5902f9dcbe89d02cd77c245c8c8423 . "./code.cg2js/cg-lisp-text.fasl")
(#xe0303327d6a318664af951b2cc408a7a . "./code.cg2js/cg-palette.fasl")
(#xb0a99bcb30278022a8814b41fc42917f . "./code.cg2js/cg-property.fasl")
(#xf6d78411850dfc42ceae86eecf29ff3a . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd4970749e27d5c0553911e26ca57bc76 . "./code.cg2js/cg-file-dialog.fasl")
(#x2b6d50b8c457f384f5c71a3cb1b6a899 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x27eddaa5eb84f9fa14645723ab88c285 . "./code.cg2js/cg-get-pixmap.fasl")
(#x796ee7ca23b5271261ca49eb2c8c37db . "./code.cg2js/cg-tray.fasl")
(#xd9019ad2ae5404df86d217a54a908560 . "./code.cg2js/cg-acache.fasl")
(#xa65c747b4e6b4fadd2e32e0af4c12d07 . "./code.cg2js/cg-message-dialog.fasl")
(#x49afb1d5f14f70d01491858863ddc03d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfa545b0505841077d4bfc8f4f8797fbe . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x5992913c6b7d4e333ed0b0138ccd44a7 . "./code.cg2js/cg-tab-control.fasl")
(#xb743e6780d5f43663f4103211c1833d3 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xdc3bbf3253eba3cac9723499c1e34518 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x5dc0b04ea963c24dfbabd5cb5338dd4c . "./code.cg2js/cg-find-string-dialog.fasl")
(#x66e0a25c0398fa38efa9515df1b1ec02 . "./code.cg2js/cg-menu.fasl")
(#x8a2fa6b15296950f0371976f23a16a40 . "./code.cg2js/cg-get-position.fasl")
(#x9363d277422343d3b260bcc611ca1f9e . "./code.cg2js/cg-trackbar.fasl")
(#x1b6c1bc2cdb3dd66fad3a1d737867b66 . "./code.cg2js/cg-selected-object.fasl")
(#xf14a6ab51157e90232058430eda0b77f . "./code.cg2js/cg-list-view.fasl")
(#x14127a52e2da7e2304bf6870a4ce7142 . "./code.cg2js/cg-html-widget.fasl")
(#xe5276f7ce2c5877b6fc6f281cbf61e5d . "./code.cg2js/cg-sample-file-menu.fasl")
(#xfe934c4412d9d858e5b1f4f59add83e5 . "./code.cg2js/cg-text-widget.fasl")
(#x89708f59f1f59a93ae0be869add65d14 . "./code.idejs/ide-base.fasl")
(#xcb94170b2dcc5e31aa6a1b3dca580103 . "./code.idejs/ide-process-browser.fasl")
(#x68b5df150549a4feca18070c4c79fe42 . "./code.idejs/ide-patch-dialog.fasl")
(#x805ef7ceb25c9f83ae25054ff390bb68 . "./code.idejs/ide-keyshort.fasl")
(#xf5d48d325eafd260ab04f74091767c2a . "./code.idejs/ide-window-list.fasl")
(#x6a8ce8d96c338ae4e489f847e223fede . "./code.idejs/ide.fasl")
(#x438e51ccc67497499b22101b7b706c44 . "./code.idejs/ide-editor.fasl")
(#xe226b25d23faec2223b6af1d2676f9a8 . "./code.idejs/ide-clipboard.fasl")
(#xed8112a19408bb503725ca9d6ce7454d . "./code.idejs/ide-trace.fasl")
(#xb92bfbe8cd17cd6a18770e7b9828abe2 . "./code.idejs/ide-class-browser.fasl")
(#xda06254859f8338536c3fc0d2fd800ed . "./code.idejs/ide-grapher.fasl")
(#x9a76aa386d67c688737baad2b03342b8 . "./code.idejs/ide-apropos.fasl")
(#x798f8ec453de06d864e3663af4293c31 . "./code.idejs/ide-navigator.fasl")
(#x9eb208b27a8234af60c99b95d6861f71 . "./code.idejs/ide-find-definitions.fasl")
(#x76aa97cc5dc3ea02f76c5efe73eba8b5 . "./code.idejs/ide-project.fasl")
(#xdd70380ca39c2417d744d8b3c9cac9b0 . "./code.idejs/ide-class-grapher.fasl")
(#x16b73175a9005d7ee1ff9c16f90a5f13 . "./code.idejs/ide-acache.fasl")
(#x21a56a352029053785a30b8bdc74f104 . "./code.idejs/ide-find-in-files.fasl")
(#xf5fa596af3958c0cb51e6aee3ba6f246 . "./code.idejs/ide-inspector.fasl")
(#xd8ef6a441c9e0977c36b0dfbc5569330 . "./code.idejs/ide-intro.fasl")
(#x8ab4847db038405423076ff77a62d4ee . "./code.idejs/ide-package-list.fasl")
(#x19f3a52de0082055e51dcef1f04eb850 . "./code.idejs/ide-dev.fasl")
(#x5fe1ed9cf68ae3d3cc79b86f0c71149a . "./code.idejs/ide-form.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
